
;**************************************************************************
;*                    AVR SPI Demo Programmer                             *
;**************************************************************************
;**************************************************************************
;*                      v 1.30 - 27.11.2017                               *
;*                                                                        *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F1829 used                                       *
;*          OSC.......: Ext. 4MHz Quarz        POWER.....:   5V extern.   *
;**************************************************************************


 INCLUDE "P16F1829.INC"


_U001			     EQU     B'00100110001100'		;see Page 48 of Datasheet

	;HS oscillator (bit2-0 = 010)
	;MCLR disabled
	;WDT enabled and controlled by SWDTEN bit of register WDTCON (bit4-3 = 01)
	;no protection
	;BOR disabled


_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	;write protection off
	;PLL enabled



	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002



;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
v_ttt		EQU	H'22'
data_tmp	EQU	H'23'
c_tmp		EQU	H'24'
kbd_buffer	EQU	H'25'
v_tst1		EQU	H'26'
v_tst2		EQU	H'27'
v_tst3		EQU	H'28'
disp_pos	EQU	H'29'

v_spi_buffi	EQU	H'2A'
v_sig1		EQU	H'2B'
v_sig2		EQU	H'2C'
v_sig3		EQU	H'2D'

v_avr_typ	EQU	H'2E'

v_add_lsb	EQU	H'30'
v_byte_low	EQU	H'31'
v_byte_high	EQU	H'32'

v_avr_pages	EQU	H'33'		;number of Flash Memory Pages
v_avr_psize	EQU	H'34'		;Size of a Page

v_page_curr	EQU	H'35'
v_word_curr	EQU	H'36'

v_ee_addh	EQU	H'37'
v_ee_addl	EQU	H'38'

v_semafor	EQU	H'39'

v_adr_MSB	EQU	H'3A'
v_adr_LSB	EQU	H'3B'
v_adr_MSB_c	EQU	H'3C'
v_adr_LSB_c	EQU	H'3D'

v_avr_demo	EQU	H'3E'
;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

		ORG	4
		goto	ir_service	;interrupt service routine

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		ORG	10


init_main	nop
		banksel	OSCCON
		movlw	B'01110000'
		movwf	OSCCON		;set 8 MHz clock

		clrf	BSR

		clrf	INTCON			;disable interrupts

		movlw	H'FF'
		movwf	PORTB

		banksel	OPTION_REG	;BANK 1
		movlw	B'01111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		movlw	H'01'
		movwf	BSR		;select BANK 1

		movlw	B'11011110'
		movwf	TRISA

		movlw	B'01111111'
		movwf	TRISB

		movlw	B'00000000'
		movwf	TRISC

;no changes required to OSCCON as after RESET the value from Config 1 word will be used (OSCCON - bit 1:bit 0 = 00)

		movlw	H'04'
		movwf	BSR		;select BANK 4
		clrf	WPUA		;disable weak pull-ups for PORTA
		clrf	WPUB		;disable weak pull-ups for PORTB
		clrf	WPUC		;disable weak pull-ups for PORTC
		bsf	WPUA,D'000'	;enable week pull-up for RA0
		bsf	WPUA,D'001'	;enable week pull-up for RA1
		bsf	WPUA,D'002'	;enable week pull-up for RA2
		bsf	WPUA,D'003'	;enable week pull-up for RA3

		movlw	H'07'
		movwf	BSR		;select BANK 7
		movlw	H'FF'
		movwf	INLVLA		;ST (1) level or TTL (0) level (?)
		movwf	INLVLB		;ST (1) level or TTL (0) level (?)
		movwf	INLVLC		;ST (1) level or TTL (0) level (?)

		banksel	LATC
		clrf	LATA
		clrf	LATC

;switch off Watch Dog
		banksel	WDTCON
		bcf	WDTCON,D'000'	;watch dog is disabled

		clrf	BSR		;select BANK 0		
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		movlw	H'FF'
		movwf	PORTB

		call	disp_init
		call	w_release

		call	dr4
		call	dr4

		call	disp_clr1
		call	disp_clr2

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		goto	main_prg


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Interrupt service routine
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

ir_service	retfie
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main entry screen
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

main_prg	bcf	INTCON,GIE

		call	i2c_init
		call	spi_init
		call	avr_prg_init
		call	read_signature

		call	w_signature

		movf	v_avr_typ,0
		btfsc	STATUS,Z
		goto	unkown_avr

		call	w_prog
;------------------------------------------------------
main_loop	call	dr2

		btfss	PORTA,D'001'		;switch S1 (Prog/Run)
		goto	avr_run

		btfss	PORTA,D'002'
		goto	avr_prog_exec		;button S2 (Exec Erase / Prog) has been pushed

		btfss	PORTA,D'003'
		goto	avr_prog_demo		;button S3 (Write Example Code) has been pushed

		goto	main_loop
;------------------------------------------------------
no_echo		call	w_no_echo
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
ne_finito	goto	ne_finito
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
unkown_avr	call	w_unkn
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
avr_run		call	avr_disconnect
		call	w_run

avr_run2	call	dr2
		btfss	PORTA,D'001'
		goto	avr_run2
		goto	main_prg
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
avr_disconnect	nop
		banksel	SSP2CON1
		bcf	SSP2CON1,D'005'		;disable SPI on MSSP2

		banksel TRISB
		bsf	TRISA,D'005'
		bsf	TRISB,D'005'
		bsf	TRISB,D'007'

		clrf	BSR

		bsf	PORTA,D'000'		;release RESET
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
avr_prg_init	bsf	PORTA,D'000'		;RESET = 1
		call	dr2
		bcf	PORTA,D'000'		;RESET = 0


		movlw	H'AC'
		call	spi_send
		movlw	H'53'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movf	v_spi_buffi,0
		xorlw	H'53'
		btfss	STATUS,Z
		goto	no_echo			;we didn't get any echo...
		movlw	H'00'
		call	spi_send

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
read_signature	movlw	H'30'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_sig1

		movlw	H'30'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'01'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_sig2

		movlw	H'30'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'02'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movf	v_spi_buffi,0
		movwf	v_sig3
;-----------------------------------------
		clrf	v_avr_typ
		clrf	v_avr_demo
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATtiny24
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n1
		movf	v_sig2,0
		xorlw	H'91'
		btfss	STATUS,Z
		goto	s1_n1
		movf	v_sig3,0
		xorlw	H'0B'
		btfss	STATUS,Z
		goto	s1_n1
;-----------------------------------------
		movlw	H'01'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'001'
		movlw	D'016'
		movwf	v_avr_psize
		movlw	D'064'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATtiny44
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n1		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n2
		movf	v_sig2,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	s1_n2
		movf	v_sig3,0
		xorlw	H'07'
		btfss	STATUS,Z
		goto	s1_n2
;-----------------------------------------
		movlw	H'02'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'001'
		movlw	D'032'
		movwf	v_avr_psize
		movlw	D'064'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATtiny84
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n2		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n3
		movf	v_sig2,0
		xorlw	H'93'
		btfss	STATUS,Z
		goto	s1_n3
		movf	v_sig3,0
		xorlw	H'0C'
		btfss	STATUS,Z
		goto	s1_n3
;-----------------------------------------
		movlw	H'03'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'001'
		movlw	D'032'
		movwf	v_avr_psize
		movlw	D'128'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATtiny13
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n3		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n4
		movf	v_sig2,0
		xorlw	H'90'
		btfss	STATUS,Z
		goto	s1_n4
		movf	v_sig3,0
		xorlw	H'07'
		btfss	STATUS,Z
		goto	s1_n4
;-----------------------------------------
		movlw	H'04'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'002'
		movlw	D'016'
		movwf	v_avr_psize
		movlw	D'032'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATtiny48
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n4		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n5
		movf	v_sig2,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	s1_n5
		movf	v_sig3,0
		xorlw	H'09'
		btfss	STATUS,Z
		goto	s1_n5
;-----------------------------------------
		movlw	H'05'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'032'
		movwf	v_avr_psize
		movlw	D'064'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATtiny88
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n5		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n6
		movf	v_sig2,0
		xorlw	H'93'
		btfss	STATUS,Z
		goto	s1_n6
		movf	v_sig3,0
		xorlw	H'11'
		btfss	STATUS,Z
		goto	s1_n6
;-----------------------------------------
		movlw	H'06'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'032'
		movwf	v_avr_psize
		movlw	D'128'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega48 / ATmega48A / ATmega48V
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n6		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n7
		movf	v_sig2,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	s1_n7
		movf	v_sig3,0
		xorlw	H'05'
		btfss	STATUS,Z
		goto	s1_n7
;-----------------------------------------
		movlw	H'07'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'032'
		movwf	v_avr_psize
		movlw	D'064'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega48PA
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n7		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n8
		movf	v_sig2,0
		xorlw	H'92'
		btfss	STATUS,Z
		goto	s1_n8
		movf	v_sig3,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	s1_n8
;-----------------------------------------
		movlw	H'08'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'032'
		movwf	v_avr_psize
		movlw	D'064'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega88 / ATmega88A / ATmega88V
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n8		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n9
		movf	v_sig2,0
		xorlw	H'93'
		btfss	STATUS,Z
		goto	s1_n9
		movf	v_sig3,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	s1_n9
;----------------------------------------
		movlw	H'09'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'032'
		movwf	v_avr_psize
		movlw	D'128'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega88PA
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n9		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n10
		movf	v_sig2,0
		xorlw	H'93'
		btfss	STATUS,Z
		goto	s1_n10
		movf	v_sig3,0
		xorlw	H'0F'
		btfss	STATUS,Z
		goto	s1_n10
;-----------------------------------------
		movlw	H'0A'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'032'
		movwf	v_avr_psize
		movlw	D'128'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega168
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n10		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n11
		movf	v_sig2,0
		xorlw	H'94'
		btfss	STATUS,Z
		goto	s1_n11
		movf	v_sig3,0
		xorlw	H'06'
		btfss	STATUS,Z
		goto	s1_n11
;-----------------------------------------
		movlw	H'0B'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'064'
		movwf	v_avr_psize
		movlw	D'128'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega168PA
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n11		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n12
		movf	v_sig2,0
		xorlw	H'94'
		btfss	STATUS,Z
		goto	s1_n12
		movf	v_sig3,0
		xorlw	H'0B'
		btfss	STATUS,Z
		goto	s1_n12
;-----------------------------------------
		movlw	H'0C'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'064'
		movwf	v_avr_psize
		movlw	D'128'
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega328
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n12		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n13
		movf	v_sig2,0
		xorlw	H'95'
		btfss	STATUS,Z
		goto	s1_n13
		movf	v_sig3,0
		xorlw	H'14'
		btfss	STATUS,Z
		goto	s1_n13
;-----------------------------------------
		movlw	H'0D'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'064'
		movwf	v_avr_psize
		movlw	D'000'			;--> 256
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega328P
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n13		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n14
		movf	v_sig2,0
		xorlw	H'95'
		btfss	STATUS,Z
		goto	s1_n14
		movf	v_sig3,0
		xorlw	H'0F'
		btfss	STATUS,Z
		goto	s1_n14
;-----------------------------------------
		movlw	H'0E'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'064'
		movwf	v_avr_psize
		movlw	D'000'			;--> 256
		movwf	v_avr_pages
		return
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
;ATmega324PA
;-----------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------
s1_n14		movf	v_sig1,0
		xorlw	H'1E'
		btfss	STATUS,Z
		goto	s1_n15
		movf	v_sig2,0
		xorlw	H'95'
		btfss	STATUS,Z
		goto	s1_n15
		movf	v_sig3,0
		xorlw	H'11'
		btfss	STATUS,Z
		goto	s1_n15
;-----------------------------------------
		movlw	H'0F'
		movwf	v_avr_typ
		bsf	v_avr_demo,D'003'
		movlw	D'064'
		movwf	v_avr_psize
		movlw	D'000'			;-> 256 pages
		movwf	v_avr_pages
		return
;-----------------------------------------
;-----------------------------------------
s1_n15		nop
s1_nx		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
avr_prog_demo	btfsc	v_avr_demo,D'001'
		goto	avr_demo01			;ATtiny24/44/84
;
		btfsc	v_avr_demo,D'002'
		goto	avr_demo02			;ATtiny13
;
		btfsc	v_avr_demo,D'003'
		goto	avr_demo03			;ATtiny48/88, ATmegas...
;------------------------------------------------------------------------------
		call	w_no_demo
		goto	eos1
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
avr_demo01	call	avr_erase
		call	w_pe
		call	avr_wecng01
		call	dr2
		call	wm_pd
		goto	eos1
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
avr_demo02	call	avr_erase
		call	w_pe
		call	dr2
		call	avr_wecng02
		call	dr2
		call	wm_pd
		goto	eos1
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
avr_demo03	call	avr_erase
		call	w_pe
		call	avr_wecng03
		call	dr2
		call	wm_pd
		goto	eos1
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;Erase Chip
avr_erase	call	w_ee

		movlw	H'AC'
		call	spi_send
		movlw	H'80'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send
		call	dr2
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
avr_prog_exec	call	avr_erase
		call	dr4
		call	dr4

		btfss	PORTA,D'002'
		goto	erase_done		

		call	w_pe
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;programming
wwp_avr		clrf	v_page_curr
		clrf	v_word_curr
		clrf	v_ee_addh
		clrf	v_ee_addl

		clrf	v_adr_MSB
		clrf	v_adr_LSB

		clrf	v_add_lsb

		clrf	v_semafor
;------------------------------------------
;open I2C communication
		call	i2c_start
		movlw	B'10100000'
		call	i2c_send
		movf	v_ee_addh,0
		call	i2c_send
		movf	v_ee_addl,0
		call	i2c_send
		call	i2c_stop

		call	i2c_start
		movlw	B'10100001'
		call	i2c_send
;------------------------------------------
w_inner_loop2	movf	v_adr_MSB,0
		movwf	v_adr_MSB_c
		movf	v_adr_LSB,0
		movwf	v_adr_LSB_c
;
w_inner_loop1	call	wow_avr			;write one word to AVR

		incf	v_add_lsb,1
		incf	v_word_curr,1

		incf	v_adr_LSB,1
		btfsc	STATUS,Z
		incf	v_adr_MSB,1

		movf	v_word_curr,0
		xorwf	v_avr_psize,0
		btfss	STATUS,Z
		goto	w_inner_loop1
;page is full ---------------------------
		call	w_progress

		btfsc	v_semafor,D'000'
		call	avr_write_page		;commit only, if there was somthing else as FFh

		clrf	v_semafor
		clrf	v_add_lsb
		clrf	v_word_curr

		incf	v_page_curr,1
		movf	v_page_curr,0
		xorwf	v_avr_pages,0
		btfss	STATUS,Z
		goto	w_inner_loop2
;----------------------------------------
;finito
		call	i2c_not_ack
		call	i2c_stop
;----------------------------------------
		call	wm_pd
		goto	eos1
erase_done	call	wm_ed
;---------------------------------------
eos1		nop
		btfsc	PORTA,D'001'
		goto	eos1

		call	disp_clr2
		goto	avr_run
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;write one word to AVR
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
wow_avr		call	i2c_receive
		movwf	v_byte_low
		call	i2c_ack
;
		call	i2c_receive
		movwf	v_byte_high
		call	i2c_ack
;
		call	load_pmp
		return
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;write one page to AVR ("commit work")
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
avr_write_page	movlw	H'4C'
		call	spi_send
		movf	v_adr_MSB_c,0
		call	spi_send
		movf	v_adr_LSB_c,0
		call	spi_send
		movlw	H'00'
		call	spi_send

		call	dr2
;------------------------------------
;Done
;------------------------------------
		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
load_pmp	movlw	H'40'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movf	v_add_lsb,0
		call	spi_send
		movf	v_byte_low,0
		call	spi_send
;
		movlw	H'48'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movf	v_add_lsb,0
		call	spi_send
		movf	v_byte_high,0
		call	spi_send

		movf	v_byte_low,0
		xorlw	H'FF'
		btfss	STATUS,Z
		bsf	v_semafor,D'000'

		movf	v_byte_high,0
		xorlw	H'FF'
		btfss	STATUS,Z
		bsf	v_semafor,D'000'

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_no_demo	movlw	B'11000000'
		call	send_comm

		call	wm_N
		call	wm_o
		call	wm_space
		call	wm_d
		call	wm_e
		call	wm_m
		call	wm_o
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_ee		movlw	B'11000000'
		call	send_comm
		call	wm_E
		call	wm_r
		call	wm_a
		call	wm_s
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_pe		movlw	B'11000000'
		call	send_comm
		call	wm_P
		call	wm_r
		call	wm_o
		call	wm_g
		call	wm_r
		call	wm_a
		call	wm_m
		call	wm_i
		call	wm_n
		call	wm_g
		call	wm_bodka
		call	wm_bodka
		call	wm_bodka
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
wm_pd		call	disp_clr2
		movlw	B'11000000'
		call	send_comm
		call	wm_D
		call	wm_o
		call	wm_n
		call	wm_e
		call	wm_bodka
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
wm_ed		call	disp_clr2
		movlw	B'11000000'
		call	send_comm
		call	wm_C
		call	wm_h
		call	wm_i
		call	wm_p
		call	wm_space
		call	wm_e
		call	wm_r
		call	wm_a
		call	wm_s
		call	wm_e
		call	wm_d
		call	wm_bodka
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
w_no_echo	movlw	B'10000000'
		call	send_comm

		call	wm_n
		call	wm_o
		call	wm_space
		call	wm_e
		call	wm_c
		call	wm_h
		call	wm_o
		return
;------------------------------------------------------------
;------------------------------------------------------------
w_signature	movlw	B'10000000'
		call	send_comm

		movf	v_avr_typ,0
		btfsc	STATUS,Z
		goto	w_stranger

		movf	v_avr_typ,0
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	w_ATtiny24

		movf	v_avr_typ,0
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	w_ATtiny44

		movf	v_avr_typ,0
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	w_ATtiny84

		movf	v_avr_typ,0
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	w_ATtiny13

		movf	v_avr_typ,0
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	w_ATtiny48

		movf	v_avr_typ,0
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	w_ATtiny88

		movf	v_avr_typ,0
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	w_ATmega48

		movf	v_avr_typ,0
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	w_ATmega48PA

		movf	v_avr_typ,0
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	w_ATmega88

		movf	v_avr_typ,0
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	w_ATmega88PA

		movf	v_avr_typ,0
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	w_ATmega168

		movf	v_avr_typ,0
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	w_ATmega168PA

		movf	v_avr_typ,0
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	w_ATmega328

		movf	v_avr_typ,0
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	w_ATmega328P

		movf	v_avr_typ,0
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	w_ATmega324PA

		goto	w_stranger
;------------------------------------------------------------
w_ATtiny24	call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_2
		call	wm_4
		return
;------------------------------------------------------------
w_ATtiny44	call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_4
		call	wm_4
		return
;------------------------------------------------------------
w_ATtiny84	call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_8
		call	wm_4
		return
;------------------------------------------------------------
w_ATtiny13	call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_1
		call	wm_3
		return
;------------------------------------------------------------
w_ATtiny48	call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_4
		call	wm_8
		return
;------------------------------------------------------------
;------------------------------------------------------------
w_ATtiny88	call	wm_A
		call	wm_T
		call	wm_t
		call	wm_i
		call	wm_n
		call	wm_y
		call	wm_8
		call	wm_8
		return
;------------------------------------------------------------
;------------------------------------------------------------
w_ATmega48	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_4
		call	wm_8
		return
;------------------------------------------------------------
w_ATmega48PA	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_4
		call	wm_8
		call	wm_P
		call	wm_A
		return
;------------------------------------------------------------
w_ATmega88	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_8
		call	wm_8
		return
;------------------------------------------------------------
w_ATmega88PA	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_8
		call	wm_8
		call	wm_P
		call	wm_A
		return
;------------------------------------------------------------
w_ATmega168	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_1
		call	wm_6
		call	wm_8
		return
;------------------------------------------------------------
w_ATmega168PA	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_1
		call	wm_6
		call	wm_8
		call	wm_P
		call	wm_A
		return
;------------------------------------------------------------
w_ATmega328	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_3
		call	wm_2
		call	wm_8
		return
;------------------------------------------------------------
w_ATmega328P	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_3
		call	wm_2
		call	wm_8
		call	wm_P
		return
;------------------------------------------------------------
w_ATmega324PA	call	wm_A
		call	wm_T
		call	wm_m
		call	wm_e
		call	wm_g
		call	wm_a
		call	wm_3
		call	wm_2
		call	wm_4
		call	wm_P
		call	wm_A
		return
;------------------------------------------------------------
w_stranger	movf	v_sig1,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_sig2,0
		movwf	kbd_buffer
		call	write_hex
		call	wm_minus

		movf	v_sig3,0
		movwf	kbd_buffer
		call	write_hex

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_unkn		movlw	B'11000000'
		call	send_comm

		call	wm_u
		call	wm_n
		call	wm_k
		call	wm_n
		call	wm_o
		call	wm_w
		call	wm_n
		call	wm_space
		call	wm_A
		call	wm_V
		call	wm_R
		return
;------------------------------------------------------------------------------
w_run		movlw	B'10001100'
		call	send_comm
		call	wm_R
		call	wm_u
		call	wm_n
		call	wm_space
		return
;------------------------------------------------------------------------------
w_prog		movlw	B'10001100'
		call	send_comm
		call	wm_P
		call	wm_r
		call	wm_o
		call	wm_g
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
w_progress	movlw	B'11000000'
		call	send_comm

		call	wm_P
		call	wm_a
		call	wm_g
		call	wm_e
		call	wm_dvojbodka
		call	wm_space
		movlw	'#'
		call	send_data

		movf	v_page_curr,0
		movwf	kbd_buffer
		call	write_dec

		call	wm_sls

		movf	v_avr_pages,0
		movwf	kbd_buffer
		call	write_dec

		call	wm_space
		call	wm_space

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x           LCD Display Communication Routines                               x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RE2 -- RA5 -- RC4
;R/W = RE1 -- N/C -- N/C
;E   = RE0 -- RA4 -- RC5

;D0 - D3 -- not connected

;D4  = RD0 -- RA0 -- RC0
;D5  = RD1 -- RA1 -- RC1
;D6  = RD2 -- RA2 -- RC2
;D7  = RD3 -- RA3 -- RC3


disp_init	nop

;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTC
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTC,D'004'
		bcf	PORTC,D'005'		;EN = 0 => disable


		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000010'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		movlw	B'00001100'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
set_LCD_D	nop
		movwf	v_ttt

		btfss	v_ttt,D'000'
		bcf	PORTC,D'000'
		call	set_0

		btfss	v_ttt,D'001'
		bcf	PORTC,D'001'
		call	set_1

		btfss	v_ttt,D'002'
		bcf	PORTC,D'002'
		call	set_2

		btfss	v_ttt,D'003'
		bcf	PORTC,D'003'
		call	set_3
		return
;-------------------------------------------
set_0		btfsc	v_ttt,D'000'
		bsf	PORTC,D'000'
		return

set_1		btfsc	v_ttt,D'001'
		bsf	PORTC,D'001'
		return

set_2		btfsc	v_ttt,D'002'
		bsf	PORTC,D'002'
		return

set_3		btfsc	v_ttt,D'003'
		bsf	PORTC,D'003'
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTC,D'005'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return

;set control bits
scbd		bsf	PORTC,D'004'		;RS -> 1 (data)
		bsf	PORTC,D'005'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTC,D'005'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTC,D'004'		;RS -> 0 (command)
		bsf	PORTC,D'005'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

dy_comm2	MOVLW   D'010'			;delay value changed: original value was: 010
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		call	d1b		;02
		call	d1b		;02
		return

d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_hviezdicka	movlw	'*'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

disp_clrx	call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		goto	disp_clrx

;-----------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
; Firmware Release
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_A
		call	wm_V
		call	wm_R
		call	wm_space
		call	wm_S
		call	wm_P
		call	wm_I
		call	wm_space
		call	wm_D
		call	wm_e
		call	wm_m
		call	wm_o
		call	wm_space
		call	wm_P
		call	wm_r
		call	wm_g

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_v
		call	wm_1
		call	wm_bodka
		call	wm_3
		call	wm_0
		call	wm_sls
		call	wm_2
		call	wm_7
		call	wm_bodka
		call	wm_1
		call	wm_1
		call	wm_bodka
		call	wm_2
		call	wm_0
		call	wm_1
		call	wm_7

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		call	wm_x

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
write_bin	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'000'
		movlw	'1'
		call	send_data

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next	movf	kbd_buffer,0
		btfsc	STATUS,Z
		goto	tst_conv_f

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn		decf	kbd_buffer,1
		goto	conv_next
;-------------------------------------------------
tst_conv_f	movf	v_tst1,0
		call	send_data
		movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; SPI communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
;set up MSSP2 for SPI BUS communication (PIC is the master)
;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
; SPI initilization on MSSP2
;
;  RB7 = SCK2
;  RB5 = SDI2
;  RA5 = SDO2
;
; ----------------------------------------------------
spi_init	nop
		banksel TRISB
		bsf	TRISB,D'005'		;SDI2 (RB5 -- input)
		bcf	TRISA,D'005'		;SDO2 (RA5 -- output)
		bcf	TRISB,D'007'		;SCK2 (RB7 -- output)

;set up SPI BUS for communication as a master for MSSP2
		banksel	APFCON1
		bsf	APFCON1,SDO2SEL		;SDO2 is on RA5 (not on RC1)

		banksel	SSP2CON1

		movlw	B'00000010'		;see Datasheet Page 260
						;bit 7   = 0    -> N/A
						;bit 6   = 0    -> N/A
						;bit 5   = 1    -> enables MSSP2
						;bit 4   = 0    -> Idle state for clock is a low level
						;bit 3-0 = 0010 -> SPI Master mode, clock = FOSC/64
		movwf	SSP2CON1

		clrf	SSP2STAT
						;bit 7   = 0 -> Input data sampled at middle of data output time
						;bit 6   = 0 -> Transmit occurs on transition from idle to active clock state

		bsf	SSP2STAT,D'006'		;bit 6   = 1 -> Transmit occurs on transition from Active to Idle clock state

		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2
		clrf	BSR
		bcf	PORTB,D'006'		;CS = 0 --> inactive
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------
spi_send	nop
		banksel	SSP2CON1
		bcf	SSP2CON1,SSPOV		;just for sure...
		bcf	SSP2STAT,BF		;just for sure...

		movwf	SSP2BUF
spil2		btfss	SSP2STAT,BF
		goto	spil2
		movf	SSP2BUF,0

		clrf	BSR
		movwf	v_spi_buffi

		return
;---------------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;I2C
;set up I2C BUS for communication as a master
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
i2c_init	nop
		banksel	TRISB
		bsf	TRISB,D'006'	;SCL
		bsf	TRISB,D'004'	;SDA

		banksel	SSP1CON1
		movlw	B'00101000'		;see Datasheet Page 277
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP1CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP1ADD
		clrf	SSP1STAT

		clrf	BSR		;select BANK 0

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
i2c_start	bsf	BSR,D'002'		;select register BANK 4
		bsf	SSP1CON2, SEN
		bcf	BSR,D'002'		;select register BANK 0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	BSR,D'002'
		bsf	SSP1CON2, PEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_ack		bsf	BSR,D'002'
		bcf	SSP1CON2, ACKDT		;ACK
		bsf	SSP1CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	BSR,D'002'
		bsf	SSP1CON2, ACKDT		;NOT ACK
		bsf	SSP1CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	BSR,D'002'
		bsf	SSP1CON2, RCEN
		bcf	BSR,D'002'
		call	d55
		bsf	BSR,D'002'
		movf	SSP1BUF,0
		bcf	BSR,D'002'
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	bsf	BSR,D'002'
		movwf	SSP1BUF
		bcf	BSR,D'002'
		call	d55			;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	BSR,D'002'
		bsf	SSP1CON2, RSEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;Example AVR Codes
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;Write
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
avr_commit	movlw	H'4C'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send
		movlw	H'00'
		call	spi_send

		call	dr2
		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;ATtiny24/44/84
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
avr_wecng01	movlw	low	avr_ex1n
		movwf	FSR0L
		movlw	high	avr_ex1n
		movwf	FSR0H

		clrf	v_add_lsb
;--------------------------------------------
avr_wecng01_24	moviw	FSR0++
		movwf	v_byte_low
;
		moviw	FSR0++
		movwf	v_byte_high
;
		call	load_pmp
;
		incf	v_add_lsb,1
		movf	v_add_lsb,0
		xorlw	D'024'
		btfss	STATUS,Z
		goto	avr_wecng01_24
;
		call	avr_commit
		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;ATtiny13
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
avr_wecng02	movlw	low	avr_ex2n
		movwf	FSR0L
		movlw	high	avr_ex2n
		movwf	FSR0H

		clrf	v_add_lsb
		goto	avr_wecng01_24
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;ATtiny88
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
avr_wecng03	movlw	low	avr_ex3n
		movwf	FSR0L
		movlw	high	avr_ex3n
		movwf	FSR0H

		clrf	v_add_lsb

		goto	avr_wecng01_24
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;ATtiny24/44/84
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
	ORG	1000

avr_ex1n	da	H'B8', H'9A'			;00-00
		da	H'C0', H'98'			;00-01
		da	H'03', H'D0'			;00-02
		da	H'C0', H'9A'			;00-03
		da	H'01', H'D0'			;00-04
		da	H'FB', H'CF'			;00-05
		da	H'1F', H'EF'			;00-06
		da	H'0F', H'EF'			;00-07
		da	H'0A', H'95'			;00-08
		da	H'F1', H'F7'			;00-09
		da	H'1A', H'95'			;00-0A
		da	H'D9', H'F7'			;00-0B
		da	H'08', H'95'			;00-0C
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;ATtiny13
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
	ORG	1100

avr_ex2n	da	H'BB', H'9A'			;00-00
		da	H'C3', H'98'			;00-01
		da	H'03', H'D0'			;00-02
		da	H'C3', H'9A'			;00-03
		da	H'01', H'D0'			;00-04
		da	H'FB', H'CF'			;00-05
		da	H'1F', H'EF'			;00-06
		da	H'0F', H'EF'			;00-07
		da	H'0A', H'95'			;00-08
		da	H'F1', H'F7'			;00-09
		da	H'1A', H'95'			;00-0A
		da	H'D9', H'F7'			;00-0B
		da	H'08', H'95'			;00-0C
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;ATtiny48/88
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
	ORG	1200

avr_ex3n	da	H'20', H'9A'			;00-00
		da	H'28', H'98'			;00-01
		da	H'03', H'D0'			;00-02
		da	H'28', H'9A'			;00-03
		da	H'01', H'D0'			;00-04
		da	H'FB', H'CF'			;00-05
		da	H'1F', H'EF'			;00-06
		da	H'0F', H'EF'			;00-07
		da	H'0A', H'95'			;00-08
		da	H'F1', H'F7'			;00-09
		da	H'1A', H'95'			;00-0A
		da	H'D9', H'F7'			;00-0B
		da	H'08', H'95'			;00-0C
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

		END